@@ -42,18 +42,20 @@ class RSATestCase(unittest.TestCase):
 
     def pp_callback(self, *args):
         # The passphrase for rsa.priv2.pem is 'qwerty'.
-        return 'qwerty'
+        return b'qwerty'
 
     def pp2_callback(self, *args):
         # Misbehaving passphrase callback.
-        pass
+        return b'blabla'
+
+    def test_rsa_exceptions(self):
+        with self.assertRaises(RSA.RSAError):
+            RSA.rsa_error()
 
     def test_loadkey_junk(self):
         with self.assertRaises(RSA.RSAError):
             RSA.load_key(self.errkey)
 
-    @unittest.skipIf(six.PY3,
-                     'test_loadkey_pp hangs under python3')
     def test_loadkey_pp(self):
         rsa = RSA.load_key(self.privkey2, self.pp_callback)
         self.assertEqual(len(rsa), 1024)
@@ -61,7 +63,6 @@ class RSATestCase(unittest.TestCase):
                          b'\000\000\000\003\001\000\001')  # aka 65537 aka 0xf4
         self.assertEqual(rsa.check_key(), 1)
 
-    @unittest.skipIf(six.PY3, 'test_loadkey_pp_bad_cp hangs under python3')
     def test_loadkey_pp_bad_cb(self):
         with self.assertRaises(RSA.RSAError):
             RSA.load_key(self.privkey2, self.pp2_callback)
@@ -181,6 +182,37 @@ class RSATestCase(unittest.TestCase):
                 b'\000\000\000\003\001\000\001',
                 b'\000\000\000\003\001')
 
+    def test_set_n(self):
+        rsa = m2.rsa_new()
+        m2.rsa_set_n(rsa, b'\000\000\000\003\001\000\001')
+
+        n = m2.rsa_get_n(rsa)
+        e = m2.rsa_get_e(rsa)
+
+        self.assertEqual(n, b'\000\000\000\003\001\000\001')
+        self.assertEqual(e, b'\x00\x00\x00\x00')
+
+    def test_set_e(self):
+        rsa = m2.rsa_new()
+        m2.rsa_set_e(rsa, b'\000\000\000\003\001\000\001')
+
+        n = m2.rsa_get_n(rsa)
+        e = m2.rsa_get_e(rsa)
+
+        self.assertEqual(e, b'\000\000\000\003\001\000\001')
+        self.assertEqual(n, b'\x00\x00\x00\x00')
+
+    def test_set_n_then_set_e(self):
+        rsa = m2.rsa_new()
+        m2.rsa_set_n(rsa, b'\000\000\000\004\020\011\006\006')
+        m2.rsa_set_e(rsa, b'\000\000\000\003\001\000\001')
+
+        n = m2.rsa_get_n(rsa)
+        e = m2.rsa_get_e(rsa)
+
+        self.assertEqual(e, b'\000\000\000\003\001\000\001')
+        self.assertEqual(n, b'\000\000\000\004\020\011\006\006')
+
     def test_newpub(self):
         old = RSA.load_pub_key(self.pubkey)
         new = RSA.new_pub_key(old.pub())
